////////////////////////////////////////////////////////////////////// 
// libsrc/PedigreeGlobals.h 
// (c) 2000-2004 Goncalo Abecasis
// 
// This file is distributed as part of the PEDSTATS source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile PEDSTATS.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Monday October 11, 2004
// 
 
#ifndef __PEDGLOBALS_H__
#define __PEDGLOBALS_H__

#include "Constant.h"
#include "StringArray.h"
#include "StringMap.h"
#include "IntArray.h"
#include "MathVector.h"

class MarkerInfo
   {
   public:
      int    chromosome;

      double position;
      double positionMale;
      double positionFemale;

      Vector freq;
      String name;
      StringArray alleleLabels;

   MarkerInfo(String & string)
      {
      serial = count++;
      name = string;
      chromosome = -1;
      position = 0.0;
      positionMale = 0.0;
      positionFemale = 0.0;
      }

   bool AdjustFrequencies();

   static int ComparePosition(MarkerInfo ** left, MarkerInfo ** right);

   String GetAlleleLabel(int allele) const;

   private:
      // How many marker info structures have we created?
      static int count;
      static String label;

      // When sorting markers, use serial_no to break ties, so
      // markers we saw first in the map file / datafile come
      // first
      int serial;
   };

class PedigreeGlobals
   {
   public:
      static int traitCount;
      static int markerCount;
      static int affectionCount;
      static int covariateCount;

      // Should be set to true if handling X-linked data
      static bool chromosomeX;
      // Set to true when map file includes position info  
      // based on sex-specific recombination fractions
      static bool sexSpecificMap;

      static StringArray  traitNames;
      static StringArray  covariateNames;
      static StringArray  affectionNames;
      static StringArray  markerNames;
      static StringIntMap markerLookup;

      // These functions are guaranteed to return a valid ID
      // If no matching attribute exists, one is created
      //

      static int GetTraitID(const char * name);
      static int GetMarkerID(const char * name);
      static int GetCovariateID(const char * name);
      static int GetAffectionID(const char * name);

      // These functions return a matching ID or -1 if none is found
      //

      static int LookupTrait(const char * name)
         { return traitNames.SlowFind(name); }
      static int LookupMarker(const char * name)
         { return markerLookup.Integer(name); }
      static int LookupCovariate(const char * name)
         { return covariateNames.SlowFind(name); }
      static int LookupAffection(const char * name)
         { return affectionNames.SlowFind(name); }

      static int markerInfoCount;
      static int markerInfoSize;
      static MarkerInfo ** markerInfo;
      static MarkerInfo ** markerInfoKey;

      static void GrowMarkerInfo();
      static MarkerInfo * GetMarkerInfo(String & name);
      static MarkerInfo * GetMarkerInfo(int marker);

      static int SortMarkersInMapOrder(IntArray & markers, int chromosome = -1);
      static void VerifySexSpecificOrder();

      static void LoadAlleleFrequencies(const char * filename);
      static void LoadAlleleFrequencies(FILE * file);

      static void LoadMarkerMap(const char * filename);
      static void LoadMarkerMap(FILE * file);

      static void LoadBasepairMap(const char * filename);
      static void LoadBasepairMap(FILE * file);

      static void WriteMapFile(const char * filename);
      static void WriteMapFile(FILE * file);

      static void WriteFreqFile(const char * filename, bool old_format = false);
      static void WriteFreqFile(FILE * file, bool old_format = false);

      PedigreeGlobals() { instanceCount++; }
      ~PedigreeGlobals();

   private:
      static int  instanceCount;

   };

#endif

 
 
 
