////////////////////////////////////////////////////////////////////// 
// libsrc/Random.h 
// (c) 2000-2004 Goncalo Abecasis
// 
// This file is distributed as part of the PEDSTATS source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile PEDSTATS.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Monday October 11, 2004
// 
 
#ifndef __RANDOM_H__
#define __RANDOM_H__

// Define a quick and dirty generator
#define RANDMUL 1664525L
#define RANDADD 1013904223L

#define RAND(seed) ((seed = seed * RANDMUL + RANDADD) & 0xFFFFFFFF)

class Random
// Implements the minimal random number generator of Park
// and Miller with Bays-Durham shuffle and added safe guards
   {
   protected:
      // values for "minimal random values"
      long  seed;
      long  last;
      long  * shuffler;

      // values for "binary random values"
      long bseed;

      // and for normal deviates
      int      normSaved;
      double   normStore;

   public:
      Random(long s = 0x7654321);
      ~Random()
         {
         delete [] shuffler;
         }


      int    Binary();     // Next bit in series of 0s and 1s
      double Next();       // Next value in series, between 0 and 1
      double Normal();     // Random number from N(0,1)
      void   Reset(long s);

      operator double()    // Random number between 0 and 1
         { return Next(); }
   };

extern Random globalRandom;

#endif
 
 
 
