////////////////////////////////////////////////////////////////////// 
// pedstats/pedstats.h 
// (c) 2000-2004 Goncalo Abecasis, 2002-2004 Jan Wigginton
// 
// This file is distributed as part of the PEDSTATS source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile PEDSTATS.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Monday October 11, 2004
// 
 
#ifndef __PEDSTATS_H__
#define __PEDSTATS_H__

#include "Constant.h"
#include "MathStats.h"
#include "MathMatrix.h"
#include "MathVector.h"
#include "MathConstant.h"
#include "Parameters.h"
#include "FilteredPedigree.h"
#include "PedigreePairs.h"
#include "Error.h"
#include "IBD.h"
#include  "PDF.h"

#include <stdio.h>

#define EXACT_MIN_ALLELES 10

void   ReadParameters(int argc, char * argv[]);
void   FamilyStatistics();

void   TraitStatistics(FILE * file, FilteredPedigree & ped, SimplePairList & pairs,
         const char * add = NULL);
void   CovarStatistics(FILE * file, FilteredPedigree & ped, SimplePairList & pairs,
         const char * add = NULL);
void   GetTraitStatistics(FILE * file, FilteredPedigree & ped, SimplePairList & pairs,
         int sex_filter, const char * label, bool is_covariate = false);

void   AffectionStatistics(FILE * file, FilteredPedigree & ped, const char * add = NULL);
void   GetAffectionStatistics(FILE * file, FilteredPedigree & ped, int sex_filter,
         const char * label);

void   MarkerStatistics(FILE * file, FilteredPedigree & ped, const char * add = NULL);
void   GetMarkerStatistics(FILE * file, FilteredPedigree & ped, int sex_filter,
         const char * label);

void   HWStatistics(FILE * f, bool foundersOnly, PDF * pdf = NULL);

int    DetermineAlleleFreqs(int m, Matrix & observed, Vector & freqs,
         IntArray & rareAlleles, int & total, int & glob, bool founders_only,
         int & smallest, int & largest);
void   DetermineAlleleNames(String & rareNames, StringArray & alleleNames, int glob,
         const Vector & freqs, const IntArray & rareAlleles);
double DetermineGenotypeCounts(const Vector & freqs, const Matrix & observed,
         int total, Matrix & real_expected, Matrix & real_observed, bool & low_counts);
int    DetermineExpectedHeterozygotes(Vector & freqs, int n_genotypes);


void   PrintHardyWeinbergHeader(FILE * f, bool founders_only);
void   PrintHardyWeinbergFooter(FILE * f, bool note, bool exact_note, bool fail,int tested_markers,
         StringArray & sparseNames, StringArray & exactSparseNames);
void   PrintHardyWeinbergResults(FILE * f, int m, double p_value, int total, int glob, 
         const Matrix & real_observed, const IntArray & rareAlleles, StringArray & alleleNames, 
         bool exact ,bool fail, bool low_counts, int smallest, int largest, int expected_hets);

double ExactHWE(int genotypes, int rare , int heterozygotes, Vector & het_probs);
double ExactTest(int total, const Vector & freqs, const Matrix & real_observed,
         double & n_rare_alleles, Vector & het_probs, bool & low_counts);

void   IBDStatistics();

void   PairStatistics(FILE * file, FilteredPedigree & ped, PedigreePairs & pairs,
         const char * add = NULL, bool traits_only = false);
void   GetPairStatistics(FILE * file, PedigreePairs & pairs,
         const char * label, bool traits_only);

// optional checks/text output
void   ByFamilyStatistics();
void   ByAffectionStatistics();
void   GetByAffectionStatistics(int a, FILE * file, const char * label, int affection_target);
void   CheckAges(PedigreePairs & pairs);
void   CheckAgeValues(int var, bool is_covariate, bool birth_year, bool max_check);
bool   RetrieveChildValue(Person * child, double & child_age, IntArray * offspring,
         IntArray * grandchildren, int var, bool is_covariate);
void   CheckChildValue(double age, double child_age, double gap, int dir, bool & first,
         Person * p, Person * child, const char * reason, const char * suffix = "");
void   UpdateAge(Pedigree & ped, Family * fam, int parent, int i, StringArray & reason,
         IntArray & extreme_child, IntArray & offspring,
         Vector & age_boundary, double gap, int dir);

void   PairTraitStatistics(FILE * file, PedigreePairs & pairs, bool is_covariate);
void   GetPairTraitStatistics(FILE * file, PedigreePairs & pairs, bool is_covariate,
         StringArray * corr_txt, IntArray * cts, bool do_unordered = true);

void   PairAffectionStatistics(FILE * file, PedigreePairs & pairs);
void   PrintDetailHeader(FILE * file, const char * additional = NULL);

void   LoadAncestors(Person * p1, IntArray & ancestors);
int    Generation(Person * p);
void   MakeIndex(IntArray & index, IntArray & counts);

// pedigree filtering functions
void   FilterPedigree(FilteredPedigree & ped);
int    BuildGenotypedFilter(FilteredPedigree & ped, IntArray & filter);
int    BuildPhenotypedFilter(FilteredPedigree & ped, IntArray & filter, bool is_covariate);
int    BuildAffectedFilter(int target_affection, FilteredPedigree & ped, IntArray & filter);
int    BuildFamilySizeFilter(FilteredPedigree & ped, IntArray & filter);

// global filtering functions
bool   FilterOnCovariate(int var, Person & p1, Person & p2);
bool   FilterOnTrait(int var, Person & p1, Person & p2);
bool   FilterOnGenotype(int marker, Person & p1, Person & p2);

bool   isParent(Person * p1, Person * p2);
bool   isOrderedParent(Person * p1, Person * p2);
bool   isGrandParentOffspring(Person * p1, Person * p2);
bool   isOrderedGrandParentOffspring(Person * p1, Person * p2);
bool   isAvuncular(Person * p1, Person * p2);
bool   isOrderedAvuncular(Person * p1, Person * p2);
bool   isOtherRelative(Person * p1, Person * p2);

bool   isCousin(Person * p1, Person * p2);
bool   isSib(Person * p1, Person * p2);
bool   isHalfSib(Person * p1, Person * p2);

bool   FilterOnBothMale(Person * p1, Person * p2);
bool   FilterOnBothFemale(Person * p1, Person * p2);
bool   FilterOnOppositeSex(Person * p1, Person * p2);

#endif





 
